//==============================================================================
// dsGrowthByAction.js
// Copyright (c) 2016 Douraku
// Released under the MIT License.
// http://opensource.org/licenses/mit-license.php
//==============================================================================

/*:
 * @plugindesc 戦闘中の行動による能力成長プラグイン ver1.02
 * @author 道楽
 *
 * @param Required Growth Exp
 * @desc 能力値成長に必要な成長値
 * @default 100
 *
 * @param Instant Apply
 * @desc 戦闘中の行動後に能力値成長を反映させるか
 * (true なら戦闘中に反映 / false なら戦闘終了時に反映)
 * @default true
 *
 * @param Apply Growth Text
 * @desc 戦闘後に成長値を反映した時に表示されるテキスト
 * %1 - 対象アクターの名前 %2 - 能力値名 %3 - 上昇量
 * @default %1 の %2 が %3 上がった！
 *
 * @param Show Growth Text Type
 * @desc 能力値成長時のテキストをどのように表示するか
 * 0 - 非表示 1 - メッセージ 2 - ポップアップ 3 - 右上一覧
 * @default 1
 *
 * @param Popup Duration
 * @desc 能力値成長時のテキストを表示する時間
 * @default 90
 *
 * @param Apply User Growth Hit
 * @desc 使用者の成長値適応をヒット時のみにするか？
 * (true ヒット時 / false 使用時)
 * @default false
 *
 * @param Hp Growth Unit
 * @desc 最大ＨＰの成長単位
 * @default 10
 *
 * @param Mp Growth Unit
 * @desc 最大ＭＰの成長単位
 * @default 10
 *
 * @param Atk Growth Unit
 * @desc 攻撃力の成長単位
 * @default 1
 *
 * @param Def Growth Unit
 * @desc 防御力の成長単位
 * @default 1
 *
 * @param Mat Growth Unit
 * @desc 魔法力の成長単位
 * @default 1
 *
 * @param Mdf Growth Unit
 * @desc 魔法防御の成長単位
 * @default 1
 *
 * @param Agi Growth Unit
 * @desc 敏捷性の成長単位
 * @default 1
 *
 * @param Luk Growth Unit
 * @desc 運の成長単位
 * @default 1
 *
 * @param Growth SE Name
 * @desc 能力値成長時の効果音のファイル名
 * @default Up4
 *
 * @param Growth SE Volume
 * @desc 能力値成長時の効果音のボリューム
 * @default 80
 *
 * @param Growth SE Pitch
 * @desc 能力値成長時の効果音のピッチ
 * @default 100
 *
 * @param Hp Growth Icon
 * @desc 最大ＨＰの成長時に表示するアイコン番号
 * 0ならアイコンを表示しない
 * @default 32
 *
 * @param Mp Growth Icon
 * @desc 最大ＭＰの成長時に表示するアイコン番号
 * 0ならアイコンを表示しない
 * @default 33
 *
 * @param Atk Growth Icon
 * @desc 攻撃力の成長時に表示するアイコン番号
 * 0ならアイコンを表示しない
 * @default 34
 *
 * @param Def Growth Icon
 * @desc 防御力の成長時に表示するアイコン番号
 * 0ならアイコンを表示しない
 * @default 35
 *
 * @param Mat Growth Icon
 * @desc 魔法力の成長時に表示するアイコン番号
 * 0ならアイコンを表示しない
 * @default 36
 *
 * @param Mdf Growth Icon
 * @desc 魔法防御の成長時に表示するアイコン番号
 * 0ならアイコンを表示しない
 * @default 37
 *
 * @param Agi Growth Icon
 * @desc 敏捷性の成長時に表示するアイコン番号
 * 0ならアイコンを表示しない
 * @default 38
 *
 * @param Luk Growth Icon
 * @desc 運の成長時に表示するアイコン番号
 * 0ならアイコンを表示しない
 * @default 39
 *
 * @param Show Status Window
 * @desc ステータス画面に蓄積率を表示するか
 * (true なら表示する / false なら表示しない)
 * @default false
 *
 * @help
 * このプラグインは以下のメモタグの設定ができます。
 *
 * -----------------------------------------------------------------------------
 * スキルに設定するメモタグ
 *
 * ・スキルの使用者が獲得できる成長値の設定
 * <userGrowthExp[能力値番号]:[成長値]>
 *  [能力値番号] - 上昇させる通常能力値の番号を1桁の数値で設定する。(数字)
 *                 0 - ＨＰ, 1 - ＭＰ, 2 - 攻撃力, 3 - 防御力
 *                 4 - 魔法力, 5 - 魔法防御, 6 - 敏捷性, 7 - 運
 *  [成長値]     - スキルを使用した時に獲得できる成長値。
 *                 この数値を一定量蓄積することで能力値が上昇する。
 *
 * ・スキルの対象者が獲得できる成長値の設定
 * <targetGrowthExp[能力値番号]:[成長値]>
 *  [能力値番号] - 上昇させる通常能力値の番号を1桁の数値で設定する。(数字)
 *                 0 - ＨＰ, 1 - ＭＰ, 2 - 攻撃力, 3 - 防御力
 *                 4 - 魔法力, 5 - 魔法防御, 6 - 敏捷性, 7 - 運
 *  [成長値]     - スキルの効果を受けた時に獲得できる成長値。
 *                 この数値を一定量蓄積することで能力値が上昇する。
 *
 * e.g.)
 *  スキルの使用者が攻撃力の成長値を「5」蓄積する
 *   <userGrowthExp2:5>
 *  スキルの対象者が運の成長値を「10」蓄積する
 *   <targetGrowthExp7:10>
 *
 *  メモタグ設定の補足：
 *  「通常攻撃」で成長値を獲得する場合はスキル1（攻撃）にタグを記述してください。
 *  「防御」で成長値を獲得する場合はスキル2（防御）にタグを記述してください。
 *   なお、対象者の成長値はスキルがヒットした時のみ上昇します。
 */

var Imported = Imported || {};
Imported.dsGrowthByAction = true;

var dsGrowthByAction = {};

(function(ns) {

	ns.Param = (function() {
		var ret = {};
		var parameters = PluginManager.parameters('dsGrowthByAction');
		ret.RequiredGrowthExp = Math.max(Number(parameters['Required Growth Exp']), 1);
		ret.InstantApply = Boolean(parameters['Instant Apply'] === 'true' || false);
		ret.ApplyGrowthText = String(parameters['Apply Growth Text']);
		ret.ShowApplyText = Boolean(parameters['Show Apply Text'] === 'true' || false);
		ret.ShowGrowthTextType = Number(parameters['Show Growth Text Type']);
		ret.PopupDuration = Number(parameters['Popup Duration']);
		ret.ApplyUserGrowthByHit = Boolean(parameters['Apply User Growth Hit'] === 'true' || false);
		ret.GrowthUnit = [];
		ret.GrowthUnit[0] = Number(parameters['Hp Growth Unit']);
		ret.GrowthUnit[1] = Number(parameters['Mp Growth Unit']);
		ret.GrowthUnit[2] = Number(parameters['Atk Growth Unit']);
		ret.GrowthUnit[3] = Number(parameters['Def Growth Unit']);
		ret.GrowthUnit[4] = Number(parameters['Mat Growth Unit']);
		ret.GrowthUnit[5] = Number(parameters['Mdf Growth Unit']);
		ret.GrowthUnit[6] = Number(parameters['Agi Growth Unit']);
		ret.GrowthUnit[7] = Number(parameters['Luk Growth Unit']);
		ret.GrowthIcon = [];
		ret.GrowthIcon[0] = Number(parameters['Hp Growth Icon']);
		ret.GrowthIcon[1] = Number(parameters['Mp Growth Icon']);
		ret.GrowthIcon[2] = Number(parameters['Atk Growth Icon']);
		ret.GrowthIcon[3] = Number(parameters['Def Growth Icon']);
		ret.GrowthIcon[4] = Number(parameters['Mat Growth Icon']);
		ret.GrowthIcon[5] = Number(parameters['Mdf Growth Icon']);
		ret.GrowthIcon[6] = Number(parameters['Agi Growth Icon']);
		ret.GrowthIcon[7] = Number(parameters['Luk Growth Icon']);
		ret.GrowthSEName = String(parameters['Growth SE Name']);
		ret.GrowthSEVolume = Number(parameters['Growth SE Volume']);
		ret.GrowthSEPitch = Number(parameters['Growth SE Pitch']);
		ret.ShowStatusWindow = Boolean(parameters['Show Status Window'] === 'true' || false);
		return ret;
	})();

	//--------------------------------------------------------------------------
	/** CONST */
	var CONST = {
		SHOW_GROWTH: {
			HIDE:    0,
			MESSAGE: 1,
			POPUP:   2,
			LIST:    3
		}
	};

	//--------------------------------------------------------------------------
	/** BattleManager */
	var _BattleManager_endAction = BattleManager.endAction;
	BattleManager.endAction = function()
	{
		if ( this._action )
		{
			this._action.applyGrowthExp();
		}
		_BattleManager_endAction.call(this);
	};

	var _BattleManager_gainRewards = BattleManager.gainRewards;
	BattleManager.gainRewards = function()
	{
		_BattleManager_gainRewards.call(this);
		this.applyGrowthParam();
	};

	BattleManager.applyGrowthParam = function()
	{
		if ( !ns.Param.InstantApply )
		{
			var delay = 0;
			$gameParty.allMembers().forEach(function(actor) {
				actor.applyGrowthParam(delay);
				if ( actor.isGrowthPopupRequested() )
				{
					delay += 20;
				}
			});
		}
	};

	//--------------------------------------------------------------------------
	/** SoundManager */
	SoundManager.playGrowthPopup = function()
	{
		var se = {
			name: ns.Param.GrowthSEName,
			volume: ns.Param.GrowthSEVolume,
			pitch: ns.Param.GrowthSEPitch,
			pan: 0,
		};
		AudioManager.playStaticSe(se);
	};

	//--------------------------------------------------------------------------
	/** Sprite_Popup */
	function Sprite_Popup()
	{
	    this.initialize.apply(this, arguments);
	}

	Sprite_Popup.prototype = Object.create(Sprite_Damage.prototype);
	Sprite_Popup.prototype.constructor = Sprite_Popup;

	Sprite_Popup.prototype.initialize = function()
	{
		Sprite_Damage.prototype.initialize.call(this);
		this.bitmap = new Bitmap(92, 20);
		this.bitmap.fontSize = 18;
		this._delay = 0;
		this._duration = ns.Param.PopupDuration;
		SoundManager.playGrowthPopup();
	};

	Sprite_Popup.prototype.setup = function(target, delay)
	{
		var growthCount = target.growthPopupCount();
		var sprite = this.createChildSpriteEx(192, 28 * growthCount);
		var width = sprite.bitmap.width;
		var height = sprite.bitmap.height;
		var lineHeight = 28;
		var line = 0;
		sprite.bitmap.paintOpacity = 192;
		sprite.bitmap.fillAll('black');
		sprite.bitmap.paintOpacity = 255;
		for ( var ii = 0; ii < 8; ii++ )
		{
			var growth = target.growthPopup(ii);
			if ( growth > 0 )
			{
				var x = 0;
				var y = line++ * lineHeight;
				var text = ns.Param.ApplyGrowthText.format(target.name(), TextManager.param(ii), growth);
				var iconIndex = ns.Param.GrowthIcon[ii];
				if ( iconIndex != 0 )
				{
					var iconBitmap = ImageManager.loadSystem('IconSet');
					var pw = Window_Base._iconWidth;
					var ph = Window_Base._iconHeight;
					var sx = iconIndex % 16 * pw;
					var sy = Math.floor(iconIndex / 16) * ph;
					var dx = sprite.bitmap.fontSize;
					var dy = sprite.bitmap.fontSize;
					sprite.bitmap.blt(iconBitmap, sx, sy, pw, ph, x + 2, y + 2, dx, dy);
					x += sprite.bitmap.fontSize + 4;
				}
				sprite.bitmap.drawText(text, x + 4, y, width - x - 4 * 2, lineHeight, 'left');
				sprite.y = 20;
			}
		}
		this.drawTargetName(target);
		this.y -= growthCount * 28;
		this._delay = delay || 0;
		this._duration += this._delay;
		this.visible = (this._delay <= 0) ? true : false;
	};

	Sprite_Popup.prototype.update = function()
	{
		Sprite_Damage.prototype.update.call(this);
		if ( this._delay > 0 )
		{
			this._delay--;
			if ( this._delay <= 0 )
			{
				this.visible = true;
			}
		}
	};

	Sprite_Popup.prototype.updateChild = function(sprite)
	{
	};

	Sprite_Popup.prototype.drawTargetName = function(target)
	{
		var width = this.bitmap.width;
		var height = this.bitmap.height;
		this.bitmap.drawText(target.name(), 1, 1, width, height, 'left');
	};

	Sprite_Popup.prototype.createChildSpriteEx = function(w, h)
	{
		var sprite = new Sprite();
		sprite.bitmap = new Bitmap(w, h);
		sprite.bitmap.fontSize = 24;
		sprite.bitmap.outlineWidth = 6;
		this.addChild(sprite);
		return sprite;
	};

	//--------------------------------------------------------------------------
	/** Sprite_Actor */
	var _Sprite_Actor_updateDamagePopup = Sprite_Actor.prototype.updateDamagePopup;
	Sprite_Actor.prototype.updateDamagePopup = function()
	{
		if ( ns.Param.ShowGrowthTextType === CONST.SHOW_GROWTH.POPUP )
		{
			this.setupGrowthPopup();
		}
		_Sprite_Actor_updateDamagePopup.call(this);
	};

	Sprite_Actor.prototype.setupGrowthPopup = function()
	{
		if ( this._battler.isGrowthPopupRequested() )
		{
			if ( this._battler.isSpriteVisible() )
			{
				var sprite = new Sprite_Popup();
				sprite.x += this._homeX + this.popupOffsetX();
				sprite.y += this._homeY + this.popupOffsetY();
				sprite.setup(this._battler, this._battler.growthPopupDelay());
				this._damages.push(sprite);
				if ( Imported.YEP_BattleEngineCore )
				{
					BattleManager._spriteset.addChild(sprite);
				}
				else
				{
					this.parent.addChild(sprite);
				}
			}
			this._battler.clearGrowthPopup();
		}
	};

	Sprite_Actor.prototype.popupOffsetX = function()
	{
		return -96;
	};

	Sprite_Actor.prototype.popupOffsetY = function()
	{
		return -64;
	};

	//--------------------------------------------------------------------------
	/** Game_Action */
	var _Game_Action_clear = Game_Action.prototype.clear;
	Game_Action.prototype.clear = function()
	{
		_Game_Action_clear.call(this);
		this._applyUser = false;
		this._appliedTargets = [];
	};

	var _Game_Action_apply = Game_Action.prototype.apply;
	Game_Action.prototype.apply = function(target)
	{
		_Game_Action_apply.call(this, target);
		if ( ns.Param.ApplyUserGrowthByHit )
		{
			if ( target.result().isHit() )
			{
				this._applyUser = true;
			}
		}
		else
		{
			this._applyUser = true;
		}
		if ( target.isActor() )
		{
			if ( target.result().isHit() )
			{
				if ( !this._appliedTargets.contains(target) )
				{
					this._appliedTargets.push(target);
				}
			}
		}
	};

	Game_Action.prototype.applyGrowthExp = function()
	{
		if ( this.subject().isActor() )
		{
			if ( this._applyUser )
			{
				this.applyUserGrowthExp();
			}
		}
		this._appliedTargets.forEach(function(target) {
			this.applyTargetGrowthExp(target);
		}, this);
	};

	Game_Action.prototype.applyUserGrowthExp = function()
	{
		var subject = this.subject();
		var item = this.item();
		for ( var ii = 0; ii < 8; ii++ )
		{
			var meta = 'userGrowthExp' + ('0'+ii).slice(-1);
			if ( item.meta[meta] )
			{
				subject.addGrowthExp(ii, Number(item.meta[meta]));
			}
		}
		if ( ns.Param.InstantApply )
		{
			subject.applyGrowthParam(0);
		}
	};

	Game_Action.prototype.applyTargetGrowthExp = function(target)
	{
		var item = this.item();
		for ( var ii = 0; ii < 8; ii++ )
		{
			var meta = 'targetGrowthExp' + ('0'+ii).slice(-1);
			if ( item.meta[meta] )
			{
				target.addGrowthExp(ii, Number(item.meta[meta]));
			}
		}
		if ( ns.Param.InstantApply )
		{
			target.applyGrowthParam(0);
		}
	};

	//--------------------------------------------------------------------------
	/** Game_Actor */
	var _Game_Actor_initMembers = Game_Actor.prototype.initMembers;
	Game_Actor.prototype.initMembers = function()
	{
		_Game_Actor_initMembers.call(this);
		this.clearGrowthParam();
	};

	Game_Actor.prototype.clearGrowthParam = function()
	{
		this._growthParam = [0,0,0,0,0,0,0,0];
		this._growthExp = [0,0,0,0,0,0,0,0];
		this._growthPopup = [0,0,0,0,0,0,0,0];
		this._growthPopupDelay = 0;
	};

	var _Game_Actor_paramPlus = Game_Actor.prototype.paramPlus;
	Game_Actor.prototype.paramPlus = function(paramId)
	{
		var param = _Game_Actor_paramPlus.call(this, paramId);
		param += this._growthParam ? (this._growthParam[paramId] || 0) : 0;
		return param;
	};

	Game_Actor.prototype.addGrowthExp = function(paramId, exp)
	{
		if ( !this._growthExp )
		{
			this.clearGrowthParam();
		}
		this._growthExp[paramId] += exp;
	};

	Game_Actor.prototype.growthPopup = function(paramId)
	{
		if ( !this._growthPopup )
		{
			this.clearGrowthParam();
		}
		return this._growthPopup[paramId];
	};

	Game_Actor.prototype.growthPopupCount = function()
	{
		if ( !this._growthPopup )
		{
			this.clearGrowthParam();
		}
		var count = 0;
		for ( var ii = 0; ii < 8; ii++ )
		{
			if ( this._growthPopup[ii] > 0 )
			{
				count++;
			}
		}
		return count;
	};

	Game_Actor.prototype.clearGrowthPopup = function()
	{
		for ( var ii = 0; ii < 8; ii++ )
		{
			this._growthPopup[ii] = 0;
		}
	};

	Game_Actor.prototype.growthPopupDelay = function()
	{
		return this._growthPopupDelay;
	};

	Game_Actor.prototype.isGrowthPopupRequested = function()
	{
		return (this.growthPopupCount() > 0) ? true : false;
	};

	Game_Actor.prototype.growthUnit = function(paramId)
	{
		return ns.Param.GrowthUnit[paramId];
	};

	Game_Actor.prototype.growthExpRate = function(paramId)
	{
		if ( !this._growthExp )
		{
			this.clearGrowthParam();
		}
		return (this._growthExp[paramId] / ns.Param.RequiredGrowthExp).clamp(0.0, 1.0);
	};

	Game_Actor.prototype.applyGrowthParam = function(delay)
	{
		if ( !this._growthExp )
		{
			this.clearGrowthParam();
		}
		var newPage = false;
		for ( var ii = 0; ii < 8; ii++ )
		{
			var add = Math.floor(this._growthExp[ii] / ns.Param.RequiredGrowthExp);
			if ( add > 0 )
			{
				var growth = this.growthUnit(ii) * add;
				this._growthPopup[ii] += growth;
				this._growthParam[ii] += growth;
				this._growthExp[ii] -= add * ns.Param.RequiredGrowthExp;
				if ( ns.Param.ShowGrowthTextType === CONST.SHOW_GROWTH.MESSAGE )
				{
					if ( !newPage )
					{
						$gameMessage.newPage();
						newPage = true;
					}
					var text = ns.Param.ApplyGrowthText.format(this.name(), TextManager.param(ii), growth);
					$gameMessage.add(text);
				}
			}
		}
		if ( newPage )
		{
			this.refresh();
		}
		this._growthPopupDelay = delay;
	};

	//--------------------------------------------------------------------------
	/** Window_Base */
	Window_Base.prototype.growthColor1 = function()
	{
		return this.textColor(28);
	};

	Window_Base.prototype.growthColor2 = function()
	{
		return this.textColor(29);
	};

	Window_Base.prototype.drawActorGrowthGauge = function(actor, paramId, x, y, width)
	{
		var color1 = this.growthColor1();
		var color2 = this.growthColor2();
		this.drawGauge(x, y, width, actor.growthExpRate(paramId), color1, color2);
	};

	//--------------------------------------------------------------------------
	/** Window_GrowthLog */
	function Window_GrowthLog()
	{
		this.initialize.apply(this, arguments);
	}

	Window_GrowthLog.prototype = Object.create(Window_Base.prototype);
	Window_GrowthLog.prototype.constructor = Window_GrowthLog;

	Window_GrowthLog.prototype.initialize = function()
	{
		var width = this.windowWidth();
		var height = this.windowHeight();
		var x = Graphics.boxWidth - width;
		Window_Base.prototype.initialize.call(this, x, 0, width, height);
		this._duration = 0;
		this.opacity = 0;
		this.createBackSprite();
		this.hide();
	};

	Window_GrowthLog.prototype.lineHeight = function()
	{
		return 28;
	};

	Window_GrowthLog.prototype.standardFontSize = function()
	{
		return 24;
	};

	Window_GrowthLog.prototype.standardPadding = function()
	{
		return 6;
	};

	Window_GrowthLog.prototype.windowWidth = function()
	{
		return 320 + this.standardPadding() * 2;
	};

	Window_GrowthLog.prototype.windowHeight = function()
	{
		return this.fittingHeight(this.maxLines());
	};

	Window_GrowthLog.prototype.maxLines = function()
	{
		return 12;
	};

	Window_GrowthLog.prototype.createBackSprite = function()
	{
		this._backSprite = new Sprite();
		this._backSprite.bitmap = new Bitmap(this.width, this.height);
		this._backSprite.opacity = 0;
		this.addChildToBack(this._backSprite);
	};

	Window_GrowthLog.prototype.update = function()
	{
		Window_Base.prototype.update.call(this);
		if ( ns.Param.ShowGrowthTextType === CONST.SHOW_GROWTH.LIST )
		{
			this.updateGrowthLog();
			this.updateDuraction();
		}
	};

	Window_GrowthLog.prototype.updateGrowthLog = function()
	{
		var lineHeight = this.lineHeight();
		var line = 0;
		$gameParty.members().forEach(function(member) {
			if ( member.isGrowthPopupRequested() )
			{
				if ( line === 0 )
				{
					this.contents.clear();
				}
				for ( var ii = 0; ii < 8; ii++ )
				{
					var growth = member.growthPopup(ii);
					if ( growth > 0 )
					{
						var x = 0;
						var y = line++ * lineHeight;
						var text = ns.Param.ApplyGrowthText.format(member.name(), TextManager.param(ii), growth);
						var iconIndex = ns.Param.GrowthIcon[ii];
						if ( iconIndex != 0 )
						{
							this.drawIcon(iconIndex, x + 2, y + 2);
							x += this.contents.fontSize + 4;
						}
						this.drawText(text, x, y, this.contentsWidth() - x, 'left');
					}
				}
				member.clearGrowthPopup();
			}
		}, this);
		if ( line > 0 )
		{
			var bitmap = this._backSprite.bitmap;
		    bitmap.clear();
		    bitmap.paintOpacity = 192;
		    bitmap.fillRect(0, this.standardPadding(), bitmap.width, lineHeight * line, 'black');
		    bitmap.paintOpacity = 255;
			this._backSprite.opacity = 255;
			this.contents.opacity = 255;
			this._duration = ns.Param.PopupDuration;
			this.show();
			SoundManager.playGrowthPopup();
		}
	};

	Window_GrowthLog.prototype.updateDuraction = function()
	{
		if ( this._duration > 0 )
		{
			this._duration--;
		}
		if ( this._duration < 10 )
		{
			var opacity = 255 * this._duration / 10;
			this._backSprite.opacity = opacity;
			this.contents.opacity = opacity;
			if ( opacity <= 0 )
			{
				this.hide();
			}
		}
	};

	Window_GrowthLog.prototype.drawIcon = function(iconIndex, x, y)
	{
		var bitmap = ImageManager.loadSystem('IconSet');
		var pw = Window_Base._iconWidth;
		var ph = Window_Base._iconHeight;
		var sx = iconIndex % 16 * pw;
		var sy = Math.floor(iconIndex / 16) * ph;
		var dx = this.contents.fontSize;
		var dy = this.contents.fontSize;
		this.contents.blt(bitmap, sx, sy, pw, ph, x, y, dx, dy);
	};

	//--------------------------------------------------------------------------
	/** Window_Status */
	Window_Status.prototype.drawActorHp = function(actor, x, y, width)
	{
		width = width || 186;
		if ( ns.Param.ShowStatusWindow )
		{
			this.drawActorGrowthGauge(this._actor, 0, x, y + 2, width);
		}
		Window_Base.prototype.drawActorHp.call(this, actor, x, y, width);
	};

	Window_Status.prototype.drawActorMp = function(actor, x, y, width)
	{
		width = width || 186;
		if ( ns.Param.ShowStatusWindow )
		{
			this.drawActorGrowthGauge(this._actor, 1, x, y + 2, width);
		}
		Window_Base.prototype.drawActorMp.call(this, actor, x, y, width);
	};

	if ( Imported.TMStatusMenuEx )
	{
		Window_Status.prototype.drawParams = function(y)
		{
			var parameters = PluginManager.parameters('TMStatusMenuEx');
			var x = +parameters['paramNameX'];
			var w = +parameters['paramNameWidth'];
			var x2 = +parameters['paramX'];
			var w2 = +parameters['paramWidth'];
			var paramBackGround = parameters['paramBackGround'] === '1' ? true : false;
			var lineHeight = this.lineHeight();
			for ( var ii = 0; ii < 6; ii++ )
			{
				var paramId = ii + 2;
				if ( TextManager.param(paramId) )
				{
					if ( paramBackGround )
					{
						this.drawParamBackGround(x, y, x2 + w2 - x);
					}
					if ( ns.Param.ShowStatusWindow )
					{
						this.drawActorGrowthGauge(this._actor, paramId, x, y, x2 + w2 - x);
					}
					this.changeTextColor(this.systemColor());
					this.drawText(TextManager.param(paramId), x, y, w);
					this.resetTextColor();
					this.drawText(this._actor.param(paramId), x2, y, w2, 'right');
					y += lineHeight;
				}
			}
		};
	}
	else
	{
		Window_Status.prototype.drawParameters = function(x, y)
		{
			var lineHeight = this.lineHeight();
			for ( var ii = 0; ii < 6; ii++ )
			{
				var paramId = ii + 2;
				var y2 = y + lineHeight * ii;
				if ( ns.Param.ShowStatusWindow )
				{
					this.drawActorGrowthGauge(this._actor, paramId, x, y2, 160+60);
				}
				this.changeTextColor(this.systemColor());
				this.drawText(TextManager.param(paramId), x, y2, 160);
				this.resetTextColor();
				this.drawText(this._actor.param(paramId), x + 160, y2, 60, 'right');
			}
		};
	}

	//--------------------------------------------------------------------------
	/** Scene_Battle */
	var _Scene_Battle_createAllWindows = Scene_Battle.prototype.createAllWindows;
	Scene_Battle.prototype.createAllWindows = function()
	{
		_Scene_Battle_createAllWindows.call(this);
		this.createGrowthWindow();
	};

	Scene_Battle.prototype.createGrowthWindow = function()
	{
		this._growthWindow = new Window_GrowthLog();
		this.addWindow(this._growthWindow);
	};

})(dsGrowthByAction);

